/*
 * Caveworld
 *
 * Copyright (c) 2014 kegare
 * https://github.com/kegare
 *
 * This mod is distributed under the terms of the Minecraft Mod Public License 1.0, or MMPL.
 * Please check the contents of the license located in http://www.mod-buildcraft.com/MMPL-1.0.txt
 */

package com.kegare.caveworld.core;

import net.minecraft.block.Block;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraftforge.common.DimensionManager;
import net.minecraftforge.common.MinecraftForge;

import com.kegare.caveworld.block.CaveBlock;
import com.kegare.caveworld.handler.CaveConnectionHandler;
import com.kegare.caveworld.handler.CaveEventHooks;
import com.kegare.caveworld.handler.CaveMusicTracker;
import com.kegare.caveworld.handler.CavePacketHandler;
import com.kegare.caveworld.handler.CavePlayerTracker;
import com.kegare.caveworld.plugin.CaveModPluginManager;
import com.kegare.caveworld.plugin.mceconomy.MCEconomyPlugin;
import com.kegare.caveworld.plugin.thaumcraft.ThaumcraftPlugin;
import com.kegare.caveworld.proxy.CommonProxy;
import com.kegare.caveworld.util.Version;
import com.kegare.caveworld.world.WorldProviderCaveworld;

import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.EventHandler;
import cpw.mods.fml.common.Mod.Metadata;
import cpw.mods.fml.common.ModMetadata;
import cpw.mods.fml.common.SidedProxy;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLPostInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.event.FMLServerStartingEvent;
import cpw.mods.fml.common.network.NetworkMod;
import cpw.mods.fml.common.registry.GameRegistry;
import cpw.mods.fml.common.registry.TickRegistry;
import cpw.mods.fml.relauncher.Side;

@Mod
(
	modid = "kegare.caveworld"
)
@NetworkMod
(
	clientSideRequired = true,
	serverSideRequired = false,
	channels = {"cw:config.sync", "cw:dim.sync", "cw:biome.sync", "cw:ore.sync", "cw:play.sound", "cw:mining.count"},
	packetHandler = CavePacketHandler.class,
	connectionHandler = CaveConnectionHandler.class
)
public class Caveworld
{
	@Metadata("kegare.caveworld")
	public static ModMetadata metadata;

	@SidedProxy(modId = "kegare.caveworld", clientSide = "com.kegare.caveworld.proxy.ClientProxy", serverSide = "com.kegare.caveworld.proxy.CommonProxy")
	public static CommonProxy proxy;

	static
	{
		CaveModPluginManager.registerPlugin(new MCEconomyPlugin());
		CaveModPluginManager.registerPlugin(new ThaumcraftPlugin());
	}

	@EventHandler
	public void preInit(FMLPreInitializationEvent event)
	{
		Version.versionCheck();

		Config.buildConfig();

		CaveBlock.initialize();
		CaveBlock.configure();

		registerRecipes();

		GameRegistry.registerPlayerTracker(new CavePlayerTracker());
	}

	private void registerRecipes()
	{
		if (Config.portalCraftRecipe)
		{
			GameRegistry.addShapedRecipe(new ItemStack(CaveBlock.portalCaveworld),
					" E ", "EPE", " D ",
					'E', Item.emerald,
					'P', Item.enderPearl,
					'D', Item.diamond
			);
		}

		if (Config.mossStoneCraftRecipe)
		{
			GameRegistry.addShapedRecipe(new ItemStack(Block.cobblestoneMossy),
					" V ", "VCV", " V ",
					'V', Block.vine,
					'C', Block.cobblestone
			);
		}
	}

	@EventHandler
	public void init(FMLInitializationEvent event)
	{
		proxy.registerRenderers();

		DimensionManager.registerProviderType(Config.dimensionCaveworld, WorldProviderCaveworld.class, true);
		DimensionManager.registerDimension(Config.dimensionCaveworld, Config.dimensionCaveworld);

		MinecraftForge.EVENT_BUS.register(new CaveEventHooks());

		TickRegistry.registerScheduledTickHandler(new CaveMusicTracker(), Side.SERVER);
	}

	@EventHandler
	public void postInit(FMLPostInitializationEvent event)
	{
		CaveBiomeManager.loadCaveBiomes();
		CaveOreManager.loadCaveOres();

		CaveModPluginManager.registerPluginContents();
	}

	@EventHandler
	public void serverStarting(FMLServerStartingEvent event)
	{
		event.registerServerCommand(new CommandCaveworld());

		if (event.getSide().isServer() && (Version.DEV_DEBUG || Config.versionNotify && Version.isOutdated()))
		{
			event.getServer().logInfo("A new Caveworld version is available : " + Version.getLatest());
		}
	}
}