/*
 * Caveworld
 *
 * Copyright (c) 2014 kegare
 * https://github.com/kegare
 *
 * This mod is distributed under the terms of the Minecraft Mod Public License 1.0, or MMPL.
 * Please check the contents of the license located in http://www.mod-buildcraft.com/MMPL-1.0.txt
 */

package com.kegare.caveworld.plugin;

import java.util.Set;
import java.util.logging.Level;

import com.google.common.collect.ImmutableSet;
import com.google.common.collect.Sets;
import com.kegare.caveworld.util.CaveLog;

import cpw.mods.fml.common.Loader;

public class CaveModPluginManager
{
	private static final Set<CaveModPlugin> CAVE_PLUGINS = Sets.newHashSet();

	public static boolean registerPlugin(CaveModPlugin plugin)
	{
		for (CaveModPlugin obj : CAVE_PLUGINS)
		{
			if (obj.pluginModId.equals(plugin.pluginModId))
			{
				return false;
			}
		}

		return CAVE_PLUGINS.add(plugin);
	}

	public static boolean unregisterPlugin(String modid)
	{
		for (CaveModPlugin plugin : CAVE_PLUGINS)
		{
			if (plugin.pluginModId.equals(modid))
			{
				return CAVE_PLUGINS.remove(plugin);
			}
		}

		return false;
	}

	public static ImmutableSet<CaveModPlugin> getModPlugins()
	{
		return new ImmutableSet.Builder<CaveModPlugin>().addAll(CAVE_PLUGINS).build();
	}

	public static Set<CaveModPlugin> getAvailableModPlugins()
	{
		Set<CaveModPlugin> plugins = Sets.newHashSet();

		for (CaveModPlugin plugin : CAVE_PLUGINS)
		{
			if (Loader.isModLoaded(plugin.pluginModId))
			{
				plugins.add(plugin);
			}
		}

		return plugins;
	}

	public static void registerPluginContents()
	{
		for (CaveModPlugin plugin : getAvailableModPlugins())
		{
			ICaveModPluginHandler handler = plugin.getPluginHandler();

			if (handler != null)
			{
				plugin.configure(false);

				try
				{
					handler.register();

					plugin.configure(true);
				}
				catch (Exception e)
				{
					CaveLog.log(Level.WARNING, e, "An error occurred trying to register %s contents", plugin.pluginModId);

					plugin.configure(false);
				}
			}
		}
	}
}